﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using Dynamicweb.Extensibility;
using Dynamicweb.Extensibility.Editors;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Ecommerce.Orders.SalesDiscounts;
using System.Web;
using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.Extensibility.AddIns;

namespace Dynamicweb.Ecommerce.LiveIntegration
{
    public class DiscountInfo
    {
        public string ParentProductID;
        public string OrderLineType;
        public double Quantity;
        public double PriceWithVAT;
        public double PriceWithoutVAT;
        public double VAT;
    }

    [AddInName("Live integration discount"),
    AddInDescription("Apply discounts retrieved via live integration")]
    public class LiveSalesDiscountProvider : SalesDiscountProvider
    {

        public override bool HideDiscountValueGroupBox
        {
            get
            {
                return true;
            }
            set
            {
                base.HideDiscountValueGroupBox = true;
            }
        }

        public override void ProcessOrder(Order order)
        {            
            if (HttpContext.Current.Session["LiveIntegrationDiscountsList" + order.Id] != null && 
                ((List<DiscountInfo>)HttpContext.Current.Session["LiveIntegrationDiscountsList" + order.Id]).Count > 0)
            {
                OrderLineCollection discounts = GetOrderLines((List<DiscountInfo>)HttpContext.Current.Session["LiveIntegrationDiscountsList" + order.Id], order);
                
                var orderedOrderlines = new OrderLineCollection();
                foreach (var ProductOrderline in order.OrderLines)
                {
                    orderedOrderlines.Add(ProductOrderline);
                    var usedOrderLines = new OrderLineCollection();
                    foreach (var orderLine in discounts)
                    {
                        if (orderLine.ParentLineId == ProductOrderline.Id)
                        {
                            orderedOrderlines.Add(orderLine);
                            usedOrderLines.Add(orderLine);
                        }
                    }
                    foreach (var item in usedOrderLines)
                    {
                        discounts.Remove(item);
                    }
                }
                foreach (var orderline in orderedOrderlines)
                {
                    if (order.OrderLines.Contains(orderline))
                        order.OrderLines.Remove(orderline);
                    order.OrderLines.Add(orderline);
                }                
            }           
        }

        public static SalesDiscount GetDiscountID()
        {
            return SalesDiscount.GetSalesDiscounts(Common.Context.Language).FirstOrDefault(sd => !string.IsNullOrEmpty(sd.DiscountType) &&
                string.Compare(sd.DiscountType, typeof(LiveSalesDiscountProvider).FullName, true) == 0);
        }

        private static OrderLineCollection GetOrderLines(List<DiscountInfo> discounts, Order order)
        {
            OrderLineCollection ret = new OrderLineCollection();
            foreach(DiscountInfo info in discounts)
            {
                var orderLine = GetOrderLine(info, order);               
                ret.Add(orderLine);
            }
            return ret;
        }

        public static OrderLine GetOrderLine(DiscountInfo discount, Order order)
        {
            var orderLine = new OrderLine();
            orderLine.Order = order;            
            orderLine.ProductName = Helpers.GetConfigValue(Constants.WebServiceConfiguration.Path, Constants.WebServiceConfiguration.DiscountText);

            orderLine.SetUnitPrice(0);
            if (discount.OrderLineType == "3" && !string.IsNullOrEmpty(discount.ParentProductID)) //1=order discount, 3=Product Discount
            {
                foreach (var productOrderLine in order.OrderLines)
                {
                    if (productOrderLine.ProductId == discount.ParentProductID)
                        orderLine.ParentLineId = productOrderLine.Id;

                }
            }
            orderLine.Quantity = discount.Quantity;
            orderLine.UnitPrice.PriceWithVAT = discount.PriceWithVAT;
            orderLine.UnitPrice.PriceWithoutVAT = discount.PriceWithoutVAT;
            orderLine.UnitPrice.VAT = discount.VAT;

            orderLine.Type = Convert.ToString(((int)OrderLineType.ProductDiscount));
            SalesDiscount liveIntegrationDiscount = GetDiscountID();
            if (liveIntegrationDiscount != null)
            {
                orderLine.DiscountId = liveIntegrationDiscount.Id;
            }
            return orderLine;
        }
    }    
}


